{% extends "admin/base_site.html" %}
{% load i18n %}
{% block content %}
  <form method="post">
    {% csrf_token %}
    {{ form.as_p }}
    <input type="submit" value="Generate Reports" class="default">
  </form>
{% endblock %}
